@@ -18,7 +18,7 @@
 """ Options module """
 
 version_string = """\
-Nuitka V0.5.16.1
+Nuitka V0.5.17
 Copyright (C) 2015 Kay Hayen."""
 
 import logging
@@ -696,6 +696,12 @@ if not positional_args:
     sys.exit("""
 Error, need positional argument with python module or main program.""")
 
+if not options.immediate_execution and len(positional_args) > 1:
+    parser.print_help()
+
+    sys.exit("""
+Error, need only one positional argument unless --run is specified.""")
+
 if options.verbose:
     logging.getLogger().setLevel(logging.DEBUG)
 else:
@@ -806,7 +812,7 @@ def shallWarnImplicitRaises():
     return options.warn_implicit_exceptions
 
 def isDebug():
-    return options.debug
+    return options.debug or options.debugger
 
 def isPythonDebug():
     return options.python_debug or sys.flags.debug
