@@ -29,7 +29,7 @@ import sys
 
 
 def getSupportedPythonVersions():
-    return ("2.6", "2.7", "3.2", "3.3", "3.4")
+    return ("2.6", "2.7", "3.2", "3.3", "3.4", "3.5")
 
 
 def getSupportedPythonVersionStr():
