@@ -35,6 +35,7 @@ from nuitka.nodes.AttributeNodes import (
 from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
 from nuitka.nodes.CallNodes import ExpressionCall, ExpressionCallNoKeywords
 from nuitka.nodes.ClassNodes import ExpressionSelectMetaclass
+from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.ComparisonNodes import ExpressionComparison
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
@@ -98,7 +99,7 @@ def _buildClassNode3(provider, node, source_ref):
 
     # This function is the Python3 special case with special re-formulation as
     # according to developer manual.
-    class_statements, class_doc = extractDocFromBody(node)
+    class_statement_nodes, class_doc = extractDocFromBody(node)
 
     # We need a scope for the temporary variables, and they might be closured.
     temp_scope = provider.allocateTempScope(
@@ -132,11 +133,50 @@ def _buildClassNode3(provider, node, source_ref):
         source_ref = source_ref
     )
 
+    if Utils.python_version >= 340 and False: # TODO: Temporarily reverted:
+        tmp_class = class_creation_function.allocateTempVariable(
+            temp_scope = None,
+            name       = "__class__"
+        )
+
+        class_target_variable_ref = ExpressionTargetTempVariableRef(
+            variable   = tmp_class,
+            source_ref = source_ref
+        )
+        class_variable_ref = ExpressionTempVariableRef(
+            variable   = tmp_class,
+            source_ref = source_ref
+        )
+    else:
+        class_variable = class_creation_function.getVariableForAssignment(
+            "__class__"
+        )
+
+        class_target_variable_ref = ExpressionTargetVariableRef(
+            variable_name = "__class__",
+            variable      = class_variable,
+            source_ref    = source_ref
+        )
+        class_variable_ref = ExpressionVariableRef(
+            variable_name = "__class__",
+            variable      = class_variable,
+            source_ref    = source_ref
+        )
+
+    code_object = CodeObjectSpec(
+        code_name     = node.name,
+        code_kind     = "Class",
+        arg_names     = (),
+        kw_only_count = 0,
+        has_starlist  = False,
+        has_stardict  = False
+    )
+
     body = buildStatementsNode(
-        provider   = class_creation_function,
-        nodes      = class_statements,
-        frame      = True,
-        source_ref = source_ref
+        provider    = class_creation_function,
+        nodes       = class_statement_nodes,
+        code_object = code_object,
+        source_ref  = source_ref
     )
 
     source_ref_orig = source_ref
@@ -227,36 +267,6 @@ def _buildClassNode3(provider, node, source_ref):
         if Utils.python_version >= 340:
             qualname_assign = statements[-1]
 
-    if Utils.python_version >= 340 and False: # TODO: Temporarily reverted:
-        tmp_class = class_creation_function.allocateTempVariable(
-            temp_scope = None,
-            name       = "__class__"
-        )
-
-        class_target_variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_class,
-            source_ref = source_ref
-        )
-        class_variable_ref = ExpressionTempVariableRef(
-            variable   = tmp_class,
-            source_ref = source_ref
-        )
-    else:
-        class_variable = class_creation_function.getVariableForAssignment(
-            "__class__"
-        )
-
-        class_target_variable_ref = ExpressionTargetVariableRef(
-            variable_name = "__class__",
-            variable      = class_variable,
-            source_ref    = source_ref
-        )
-        class_variable_ref = ExpressionVariableRef(
-            variable_name = "__class__",
-            variable      = class_variable,
-            source_ref    = source_ref
-        )
-
     statements += [
         body,
         StatementAssignmentVariable(
@@ -319,6 +329,7 @@ def _buildClassNode3(provider, node, source_ref):
                 function_body = class_creation_function,
                 source_ref    = source_ref
             ),
+            code_object  = code_object,
             defaults     = (),
             kw_defaults  = None,
             annotations  = None,
@@ -552,11 +563,8 @@ def _buildClassNode3(provider, node, source_ref):
     if Utils.python_version >= 340:
         class_assign = statements[-1]
 
-        # assert False, class_creation_function
         class_creation_function.qualname_setup = class_assign, qualname_assign
 
-
-
     final = (
         StatementReleaseVariable(
             variable   = tmp_bases,
@@ -585,10 +593,9 @@ def _buildClassNode3(provider, node, source_ref):
 
 
 def _buildClassNode2(provider, node, source_ref):
-    class_statements, class_doc = extractDocFromBody(node)
-
     # This function is the Python2 special case with special re-formulation as
-    # according to developer manual.
+    # according to developer manual, and it's very detailed, pylint: disable=R0914
+    class_statement_nodes, class_doc = extractDocFromBody(node)
 
     function_body = ExpressionClassBody(
         provider   = provider,
@@ -599,13 +606,21 @@ def _buildClassNode2(provider, node, source_ref):
         source_ref = source_ref
     )
 
-    body = buildStatementsNode(
-        provider   = function_body,
-        nodes      = class_statements,
-        frame      = True,
-        source_ref = source_ref
+    code_object = CodeObjectSpec(
+        code_name     = node.name,
+        code_kind     = "Class",
+        arg_names     = (),
+        kw_only_count = 0,
+        has_starlist  = False,
+        has_stardict  = False
     )
 
+    body = buildStatementsNode(
+        provider    = function_body,
+        nodes       = class_statement_nodes,
+        code_object = code_object,
+        source_ref  = source_ref
+    )
     if body is not None:
         # The frame guard has nothing to tell its line number to.
         body.source_ref = source_ref.atInternal()
@@ -698,6 +713,7 @@ def _buildClassNode2(provider, node, source_ref):
                         function_body = function_body,
                         source_ref    = source_ref
                     ),
+                    code_object  = None,
                     defaults     = (),
                     kw_defaults  = None,
                     annotations  = None,
